
#include <stdlib.h>
#include <stdio.h>

#include "cgdk.h"
#include "spr.h"
#include "svga.h"
#include "pal.h"
#include "car.h"
#include "keyb.h"
#include "tm.h"
#include "timer.h"

TRGBColor pal [256];

static void CSFVIEW_printInfo (TCSF *spr)
{
  int i;

  printf ("Description                  : %s\n",spr->desc);
//  printf ("File version                 : %i.%i\n",hil (spr->version), lol (spr->version));
  printf ("Count of sprites             : %i\n",spr->count);
  printf ("Size X                       : %i\n",spr->sizeX);
  printf ("Size Y                       : %i\n",spr->sizeY);
  printf ("Bytes per pixel              : %i\n",spr->bytesPerPixel);
  printf ("Total colors                 : %i\n",spr->maxColor);
  printf ("File type                    : %s\n",(spr->maxChar) ? "Font" : "Sprite");
  if (spr->maxChar)
  {
    printf ("Characters count             : %i\n",spr->maxChar);
    printf ("Defined characters           : ");
    for (i = 0; i <= 255; i++)
    {
      if (spr->charTable [i] != 0)
      {
        putchar (i);
      }
    }
    printf ("\n");
    printf ("Default characters X spacing : %i\n",spr->xSpacing);
    printf ("Default characters Y spacing : %i\n",spr->ySpacing);
  }
}

int main (void)
{
  TCSF *spr;
  TCAR *carHandle;
  int i,j,k;
  int videoMode;

  printf ("\n"
          "CSFVIEW - Cybertek Sprite/Font viewer\n"
          "\n%s\n",CGDK_copyright);

  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  if ((_argc == 1+1) || (_argc == 2+1))
  {
    carHandle = CAR_open (CAR_EMULATION);
    if ((spr = CSF_new (carHandle, _argv [1])) != NULL)
    {
      CSFVIEW_printInfo (spr);
      CSF_updateColors (spr, pal,3,253);

      if (SVGA_compatible & VGA_COMPATIBLE)
      {
//        TM_init ();
        if (_argc == 2+1)
          videoMode = strtol (_argv [2], NULL, 0);
        else
          videoMode = SVGA_recommendVideoMode (spr->sizeX,spr->sizeY, 1, VMODE_ANY);

        if (SVGA_setVideoMode (videoMode) != SVGA_SETMODE_OK)
        {
          CGDK_exit ("Unable to set the specified videomode.\n", EXIT_FAILURE);
        }
        PAL_clear (0, 256);
        SVGA_clear (1);
        pal [1].b = 255;
        pal [255].r = 255;

        k = 0;
        for (i = 0; i <= SVGA_maxY / spr->sizeY; i++)
        {
          for (j = 0; j <= SVGA_maxX / spr->sizeX; j++)
          {
            CSF_put (spr, spr->sizeX*j, spr->sizeY*i, k % spr->count);
            k++;
          }
        }
        SVGA_flush ();

        SVGA_putStr (16,SVGA_maxY-32,"Press any key to continue ...",255);

        PAL_fadeIn (pal, 100, 0, 256);
        KEYB_init ();
        KEYB_wait ();
//      PAL_fadeOut (pal);
        KEYB_done ();

        SVGA_done ();
      }
      else
      {
        CGDK_exit ("This program requires a VGA compatible videocard.\n", EXIT_FAILURE);
      }

      CSF_delete (spr);
    }
    else
    {
      sprintf (exitStr, "Cybertek Sprite/Font %s not found.\n",_argv [1]);
      CGDK_exit (exitStr, EXIT_FAILURE);
    }
  }
  else
  {
    CGDK_exit ("Syntax is: CSFVIEW <filename.spr> [video_mode]\n", EXIT_FAILURE);
  }
  return 0;
}
