
#include <stdlib.h>
#include <mem.h>
#include <stdio.h>
#include <string.h>

#include "cgdk.h"
#include "car.h"
#include "pal.h"
#include "dith.h"
#include "file.h"
#include "mir.h"

#include "svga.h"
#include "vsb.h"

#define DITH_LINELEN 256

#ifdef __TURBOC__
  #define DITH_LINES 10
  #define DITH_PLUS 0
#else
  #define DITH_LINES 256
  #define DITH_PLUS 65536
#endif

char *DITH_shadeBuf;
TDITH_tab DITH_tab;

void DITH_done (void)
{
  if (DITH_tab.buf != NULL)
  {
    _free (DITH_tab.buf);
    DITH_tab.buf = NULL;

    DITH_shadeBuf = NULL;
  }
}

static void DITH_setupShadeBuf (void)
{
  DITH_shadeBuf = DITH_tab.buf;

  #ifndef __TURBOC__
  if ( (*(int *)&DITH_shadeBuf) & 0x0000FFFF)
  {
    *(int *)&DITH_shadeBuf = ((*(int *)&DITH_shadeBuf) & 0xFFFF0000) + 0x10000;
  }
  #endif
}


int DITH_init (int maxSteps)
{
  unsigned i, j;
//  unsigned offset;

  strcpy (DITH_tab.desc, DITH_DESC);
  DITH_tab.id = DITH_ID;
  strcpy (DITH_tab.label, "");
  DITH_tab.version = DITH_VERSION;

  if (sizeof (TDITH_tab) == DITH_HEADERSIZE)
    ;
  else
    return -1;

  DITH_done ();

  DITH_tab.buf = (char *) _calloc ((unsigned)(DITH_LINES*DITH_LINELEN)+DITH_PLUS,1 ,NULL);

  if (DITH_tab.buf != NULL)
  {
    DITH_setupShadeBuf ();

    DITH_tab.maxStep = maxSteps;

 //   offset = 0;
    for (i = 0; i < (unsigned)DITH_LINES*DITH_LINELEN; i++)
    {
      DITH_shadeBuf [i] = i;
    }
    return 0;
  }
  else
  {
    return -2;
  }
}


int DITH_save (char *fileName)
{
  int handle;

  if (DITH_tab.buf != NULL)
  {
    if (F_write (fileName, &DITH_tab, sizeof (TDITH_tab)) == 0)
    {
      if (F_append (fileName, DITH_shadeBuf, (unsigned)DITH_LINELEN*DITH_LINES)==0)
      {
        return 0;
      }
      else
      {
        return 3;
      }
    }
    else
    {
      return 2;
    }
  }
  else
  {
    return 1;
  }
}

int DITH_load (TCAR *carHandle, char *fileName)
{
  unsigned bufSize;

  if (CAR_seek (carHandle, fileName, 0) != -1)
  {
    DITH_done ();
    if (CAR_read (carHandle, &DITH_tab, sizeof (TDITH_tab)) == sizeof (TDITH_tab))
    {
      if (DITH_tab.version <= DITH_VERSION)
      {
        bufSize = (unsigned)DITH_LINES*DITH_LINELEN+DITH_PLUS;
        DITH_tab.buf = (char *) _alloc ((unsigned)bufSize);
        if (DITH_tab.buf != NULL)
        {
          DITH_setupShadeBuf ();

          if (CAR_read (carHandle, DITH_shadeBuf, (unsigned)DITH_LINES*DITH_LINELEN) == DITH_LINES*DITH_LINELEN)
          {
            return 0;
          }
          else
          {
            DITH_done ();
          }
        }
      }
    }
  }
  return 1;
}

void DITH_fill (TRGBColor *pal, int baseColor, int maxColor,
                  int lightSmooth, int lightShift)
{
  int i,j;

  TRGBColor fadeColor;
  TRGBColor shinyColor;
  TRGBColor darkColor;

  if (DITH_tab.buf != NULL)
  {
    shinyColor.r = shinyColor.g = shinyColor.b = 255;
    darkColor.r = 0;
    darkColor.g = 0;
    darkColor.b = 0;

    DITH_tab.maxColor = (unsigned char) maxColor;
    DITH_tab.baseColor = (unsigned char) baseColor;

    for (j = 0; j < DITH_tab.maxColor; j++)
    {
      for (i = 0; i < DITH_tab.maxStep; i++)
      {
        PAL_colorFadeStep (&fadeColor, pal [baseColor+j], shinyColor, i+lightShift*(i != 0), lightSmooth);
        DITH_set (baseColor+j, i, PAL_nearestColor (fadeColor, pal+baseColor, maxColor)+baseColor);

        #ifndef __TURBOC__
        PAL_colorFadeStep (&fadeColor, pal [baseColor+j], darkColor, i+lightShift*(i != 0), lightSmooth);
        DITH_set (baseColor+j, -i, PAL_nearestColor (fadeColor, pal+baseColor, maxColor)+baseColor);
        #endif
      }
    }
  }
}

#if defined __TURBOC__ || defined CGDK_TRIAL
void DITH_shadeMap (void *raw, int rawSizeX, void *dstRaw, int dstFormatX,
                    void *spr, int srcSizeX, int srcSizeY)
{
  unsigned i;
  char *dst = (char *) dstRaw;
  char *src = (char *) spr;
  char *msk = (char *) raw;

  for ( ; srcSizeY > 0; srcSizeY--)
  {
    for (i = 0; i < srcSizeX; i++)
    {
//      if (src [i])
      {
        dst [i] = DITH_get (src [i], msk [i]);
      }
    }
    dst += dstFormatX;
    src += srcSizeX;
    msk += rawSizeX;
  }
}

void DITH_shadeRaw (void *mask, int srcFormatX, void *dstRaw, int dstFormatX,
                     int srcSizeX, int srcSizeY)
{
  unsigned i;
  char *dst = (char *) dstRaw;
  char *src = (char *) dstRaw;
  char *msk = (char *) mask;

  for ( ; srcSizeY > 0; srcSizeY--)
  {
    for (i = 0; i < srcSizeX; i++)
    {
//      if (src [i])
      {
        dst [i] = DITH_get (src [i], msk [i]);
      }
    }
    dst += dstFormatX;
    src += dstFormatX;
    msk += srcFormatX;
  }
}

void DITH_shadeMask (int mask, char *dstRaw, int dstFormatX,
                     char *srcRaw, int srcFormatX, int srcSizeX, int srcSizeY)
{
  unsigned i;

  for ( ; srcSizeY > 0; srcSizeY--)
  {
    for (i = 0; i < srcSizeX; i++)
    {
//      if (srcRaw [i])
      {
        dstRaw [i] = DITH_get (srcRaw [i], mask);
      }
    }
    dstRaw += dstFormatX;
    srcRaw += srcFormatX;
  }
}
#endif

void DITH_shadeScreen (char shade)
{
  MIRL_restoreAll ();
  VSB_getWin (VSB_sbuf, 0, 0, SVGA_maxX, SVGA_maxY, SVGA_maxX);
  DITH_shadeMask (shade, VSB_sbuf, SVGA_maxX, VSB_sbuf, SVGA_maxX, SVGA_maxX, SVGA_maxY);
  VSB_putWin (VSB_sbuf, 0, 0, SVGA_maxX, SVGA_maxY, SVGA_maxX);
  SVGA_wait (1);
  SVGA_putWinNoVSB (VSB_sbuf, 0, 0, SVGA_maxX, SVGA_maxY, SVGA_maxX);
}
