
/* UNIT XVGA_IO VERSION 1.03, (C) Cerberus Electronics 1995 */
/* ======================================================== */
/*                                                          */
/* Changes: XVGA_PutSPR fixed                               */
/*          XVGA_PutSPR2 no longer available.               */
/*          XVGA_GetSPR added.                              */
/*          XVGA_Init (int virtualX, virtualY, RealX, RealY)*/
/*            requires total count of pixel collumns.       */
/*            (320+count of extra collumns.)                */

#include <dos.h>
#include <stdlib.h>
#include <mem.h>
#include <conio.h>

#include "svga.h"
#include "dpmi.h"
//#include "cgdk.h"
#include "vsb.h"
#include "tm.h"
#include "xvga.h"


int XVGA_readPlane;
int XVGA_writePlane;

int XVGA_columns;

static char XVGA_VDMA;
static int XVGA_Chipset=-1;

#include "x256_.cpp"

#define XVGA_ERR_OK 0
#define XVGA_ERR_NOVGA 1
#define XVGA_ERR_UNKNOWNMODE 2
#define XVGA_ERR_BADVIRTUALX 3

#define XVGA_VGA 0
#define XVGA_ET3000 1
#define XVGA_ET4000 2

#define XVGA_X_PLUSMAX 16
#define XVGA_Y_PLUSMAX 15


static void Set_RowOffset (int Offset)
{
  int Port = CRTC_ADDR;
  int a;

  outp (Port, 0x07);
  a = inp (Port+1);
  outp (Port+1, resetBit (a,4));

  outp (Port, 0x09);
  a = inp (Port+1);
  outp (Port+1, resetBit (a,6));

  outp (Port, 0x18);
  outp (Port+1, Offset);
}

static void Set_VerticalTotal (int Lines)
{
  int Port = CRTC_ADDR;
  int a;

  outp (Port, 0x07);
  a = inp (Port+1);
  outp (Port+1, resetBit (a,5));

  outp (Port, 0x07);
  a = inp (Port+1);
  outp (Port+1, resetBit (a,0));

  outp (Port, 0x06);
  outp (Port+1, Lines);
}


/*
static void ET4000_SetBank (int a)
{
  outp (0x3CD, (a << 4) | a);
}

static void ET3000_SetBank (int a)
{
  outp (0x3CD, (a << 3) | a);
}

static void VGA_SetBank (int a)
{

}


typedef void (*SetBankFunc) (int);

static SetBankFunc SetBank [] = {VGA_SetBank, ET3000_SetBank, ET4000_SetBank};
#define maxchipset 2

void ET4000_Proc2 (void)
{
  outp (0x3BF, 0x03);
  outp (0x3D8, 0xA0);
}

void ET4000_Proc3 (int A)
{
  outpw (0x3D4, ((A << 8) & 3) + 0x33);
}
*/

#include "et_.cpp"




static int XVGA_detectChipset (void)
{
  void *stateBuf;

  if (XVGA_Chipset == -1)
  {
    stateBuf = VGA_saveState ();
    XVGA_Chipset = ET_detect ();
    VGA_restoreState (stateBuf);
  }

  return XVGA_Chipset;
}

/*
void XVGA_initRegs (int Mode)
{
  char VDMA = XVGA_VDMA;
  int a;

  // Turn off the Chain-4 bit (bit 3 at index 4, port 0x3c4):
  outp (SEQU_ADDR, 0x4);
  a = inp (SEQU_ADDR+1);
  SetNBit (&a, 2);
  ResetNBit (&a, 3);
//  outp (SEQU_ADDR+1, a);

  // Turn off doubleword mode, by setting the Underline Location
  // register (index 0x14, port 0x3d4):
  outp (CRTC_ADDR, 0x14);
  a = inp (CRTC_ADDR+1);
  ResetNBit (&a, 6);
//  outp (CRTC_ADDR+1, a);

  // Turn off word mode, by setting the Mode Control register
  // of the CRT Controller (index 0x17, port 0x3d4):
  outp (CRTC_ADDR, 0x17);
  a = inp (CRTC_ADDR+1);
  SetNBit (&a, 6);
//  outp (CRTC_ADDR+1, a);

  // set the total number of column per line
  outp (CRTC_ADDR, 0x13);
  outp (CRTC_ADDR+1, VDMA);
}
*/



void XVGA_scroll (void)
{
  int pos;
  int panning;
  int curX, curY;

  if (VESA_MATROXUSE)
  {
    curX = SVGA_curX;
    curY = SVGA_curY;
  }
  else
  {
    curX = SVGA_curXNext;
    curY = SVGA_curYNext;
  }

  pos = (curX + (long)curY * SVGA_virtualX)/4;
  panning = (curX % 4) <<1;

  DPMI_disableInt ();

  outp (CRTC_ADDR, 0xC);
  outp (CRTC_ADDR+1, his (pos));
  outp (CRTC_ADDR, 0xD);
  outp (CRTC_ADDR+1, los (pos));

  outp (0x3C0, 0x20+0x13);
  outp (0x3C0, panning);

  DPMI_enableInt ();
}

/*
void XVGA_scroll (void)
{
  int pos;
  int panning;

  pos = (SVGA_curX + (long)SVGA_curY * SVGA_virtualX)/4;
  panning = (SVGA_curX % 4) <<1;

  DPMI_disableInt ();

  outp (CRTC_ADDR, 0xC);
  outp (CRTC_ADDR+1, his (pos));
  outp (CRTC_ADDR, 0xD);
  outp (CRTC_ADDR+1, los (pos));

  outp (0x3C0, 0x20+0x13);
  outp (0x3C0, panning);

  DPMI_enableInt ();
}
*/

#include "xmode_.cpp"

static void XVGA_initRegs (int mode)
{
  int i;
  int value;

  //set the current graphics mode flags
  outpw( SC_INDEX, 0x0604 );
  outpw( SC_INDEX, 0x0100 );

  //Set the dot clock scanning rate, if applicable
  if ( xSetup [mode]->dotClock != 0 )
  {
    outp( MISC_OUTPUT, xSetup [mode]->dotClock );
  }
  //Undo reset (restart sequencer)
  outpw( SC_INDEX, 0x0300 );
  //Reprogram the CRT controller.  The VSync End register contains
  //the register write-protect bit, so we save the old value, mask
  //out the write-protect bit, and rewrite the value
  outp( CRTC_ADDR, 0x11 );
  value = inp( CRTC_ADDR + 1 );
  outp( CRTC_ADDR, 0x11 );
  outp( CRTC_ADDR + 1, value & 0x07f );
  //now that we've removed the write-protection, we reset all the
  //appropriate CRT registers

  for (i = 0; i < MAX_CRTC_REGISTER; i++)
  {
    value = xSetup [mode]->registers [i];
    if (value)
    {
      outpw( CRTC_ADDR, value);
    }
  }

  // set the total number of column per line
  outp (CRTC_ADDR, 0x13);
  outp (CRTC_ADDR+1, XVGA_VDMA);
}

int XVGA_setVideoMode (int xMode)
{
  int mode;

  xMode = xMode ^ XVGA;
  mode = los (xMode);

  VGA_setVideoMode (VGA_320x200x256);

  CRTC_ADDR = CRTC_ADDR;
  STATUS_ADDR = STATUS_ADDR;

  SVGA_maxX = xSetup [mode]->width;
  SVGA_maxY = xSetup [mode]->height;
  SVGA_max = (unsigned)SVGA_maxX*SVGA_maxY;

  if ((xMode & NO_SCROLL) == 0)
  {
    SVGA_doubleBuffer = (xMode & DOUBLE_BUFFER) != 0;
    if (SVGA_doubleBuffer == 0)
    {
      SVGA_doubleBuffer = (XVGA_detectChipset () != 0);
    }

    SVGA_maxXPlus = XVGA_X_PLUSMAX;
    SVGA_maxYPlus = XVGA_Y_PLUSMAX;

    if (SVGA_doubleBuffer)
    {
      SVGA_virtualX = SVGA_maxX+SVGA_maxXPlus;
      if (SVGA_virtualX % 8)
        SVGA_virtualX+=4;
      SVGA_virtualY = SVGA_maxY+SVGA_maxYPlus;
      SVGA_scanLines = 2*SVGA_virtualY;
    }
    else
    {
      SVGA_virtualX = 512;
      SVGA_virtualY = 512;
      SVGA_scanLines = SVGA_virtualY;
    }
  }
  else
  {
    SVGA_doubleBuffer = 0;
    SVGA_virtualX = SVGA_maxX;
    SVGA_virtualY = SVGA_maxY;
    SVGA_scanLines = SVGA_maxY;
    SVGA_maxXPlus = 0;
    SVGA_maxYPlus = 0;
  }

  XVGA_VDMA = SVGA_virtualX/8;
  XVGA_columns = SVGA_virtualX/4;

  SVGA_curX = SVGA_curY = 0;

  XVGA_initRegs (mode);
  SVGA_linearBuf     = NULL;
  SVGA_palType       = PAL_6BIT;
  SVGA_bytesPerPixel = 1;
  SVGA_graph = 1;

  SVGA_virtual = (long)SVGA_virtualX*SVGA_virtualY;
  return SVGA_SETMODE_OK;
}


int XVGA_videoModeSupported (int videoMode)
{
  int mode;
  int maxX;
  int maxY;
  int virtualX;
  int virtualY;
  int maxPages;
  int doubleBuffer;
  long mem;
  long memRequired;

  if (videoMode & XVGA)
  {
    mode = los (videoMode);
    if (mode < LAST_X_MODE)
    {
      if (SVGA_compatible & VGA_COMPATIBLE)
      {
        doubleBuffer = videoMode & DOUBLE_BUFFER;

        if ((videoMode & NO_SCROLL) == 0)
        {
          maxX = xSetup [mode]->width;
          maxY = xSetup [mode]->height;

          if (!doubleBuffer)
          {
            doubleBuffer = (XVGA_detectChipset () != 0);
          }
          virtualX = maxX + XVGA_X_PLUSMAX;
          virtualY = maxY + XVGA_Y_PLUSMAX;

          if (doubleBuffer)
          {
            mem = 128*1024L;
          }
          else
          {
            if ((virtualX > 512) || (virtualY > 512))
            {
              return 3;
            }
            mem = 256*1024L;
          }
          memRequired = ((long)virtualX*virtualY);
          return memRequired > mem;
        }
        else
        {
          if (doubleBuffer)
            return 1;
          else
            return 0;
        }
      }
    }
  }
  return 2;
}


int XVGA_getModeInfo (int mode, int *xRes, int *yRes,
                          int *bytesPerLine,int *bitsPerPixel)
{
  if (XVGA_videoModeSupported (mode)==0)
  {
    mode &= ~(XVGA|DOUBLE_BUFFER);
    *xRes = xSetup [mode]->width;
    *yRes = xSetup [mode]->height;
    *bytesPerLine = *xRes/4;
    *bitsPerPixel = 8;
    return 0;
  }
  return -1;
}


/*
int XVGA_installed (void)
{
  return VGA_installed ();
}
*/


/*

SplitScreen     proc
     mov       dx,03D4h
     mov       bx,ax
     mov       ah,bl
     mov       al,18h
     out       dx,ax
     mov       al,07h
     out       dx,al
     inc       dx
     in        al,dx
     and       al,11101111b
     mov       bl,bh
     and       bl,00000001b
     mov       cl,4
     shl       bl,cl
     or        al,bl
     out       dx,al
     dec       dx
     mov       al,9h
     out       dx,al
     inc       dx
     in        al,dx
     and       al,10111111b
     and       bh,00000010b
     mov       cl,5
     shl       bh,cl
     or        al,bh
     out       dx,al
     ret
SplitScreen       endp


*/




void XVGA_raw2Planar (void *Dst, void *Src, int SIZE_X, int SIZE_Y)
{
  int i, j, k = 0;
  char *_Dst = (char *) Dst;
  char *_Src = (char *) Src;
  int LineSize = (SIZE_X*SIZE_Y) >> 2;

  for (j = 0; j < 4; j++)
  {
    for (i = 0; i < LineSize; i++, k++)
    {
      _Dst [k] = _Src [4*i + j];
    }
  }
}


void XVGA_planar2Raw (void *Dst, void *Src, int SIZE_X, int SIZE_Y)
{
  int i, j, k = 0;
  char *_Dst = (char *) Dst;
  char *_Src = (char *) Src;
  int LineSize = (SIZE_X*SIZE_Y) >> 2;

  for (j = 0; j < 4; j++)
  {
    for (i = 0; i < LineSize; i++, k++)
    {
      _Dst [4*i + j] = _Src [k];
    }
  }
}

void XVGA_setWritePlane (int plane)
{
  if (plane != XVGA_writePlane)
  {
    outp (SEQU_ADDR, 2);
    outp (SEQU_ADDR+1, 1 << plane);
    XVGA_writePlane = plane;
    SVGA_setBankCount++;
  }
}


void XVGA_setReadPlane (int plane)
{
//  int a;

  if (plane != XVGA_readPlane)
  {
    outp (0x03CE, 4);
//    a = inp (0x03CE+1);
//    a = resetBit (a, 0);
//    a = resetBit (a, 1);
//    a &= 0xff-3;
//    plane &= 3;
    outp (0x3CE+1, (plane & 3) | resetBit (resetBit (inp (0x3CE+1),0), 1));
    XVGA_readPlane = plane;
  }
}

